@@ -1,9 +1,6 @@
 # Just some routines related to the Memoize module that are used in
 # more than one place in XMLTV.  But not general enough to merge back
 # into Memoize.
-#
-# $Id: Memoize.pm,v 1.14 2005/02/06 20:18:12 epaepa Exp $
-#
 
 package XMLTV::Memoize;
 use strict;
@@ -53,8 +50,12 @@ sub check_argv( @ ) {
     my $p = new Getopt::Long::Parser(config => ['passthrough']);
     die if not $p;
     my $opt_cache;
-    my $result = $p->getoptions('cache:s' => \$opt_cache);
+    my $opt_quiet = 0;
+    my $result = $p->getoptions('cache:s' => \$opt_cache,
+                                'quiet' => \$opt_quiet );
     die "failure processing --cache option" if not $result;
+    unshift @ARGV, "--quiet" if $opt_quiet;
+
     return undef if not defined $opt_cache;
     my $filename;
     if ($opt_cache eq '') {
@@ -65,13 +66,14 @@ sub check_argv( @ ) {
     else {
 	$filename = $opt_cache;
     }
-    print STDERR "using cache $filename\n";
+    print STDERR "using cache $filename\n" unless $opt_quiet;
 
     require POSIX;
     require Memoize;
     require DB_File;
-    my @tie_args = ('DB_File', $filename,
-		    POSIX::O_RDWR() | POSIX::O_CREAT(), 0666);
+    # Annoyingly tie(%cache, @tie_args) doesn't work
+    #my @tie_args = ('DB_File', $filename,
+    #		    POSIX::O_RDWR() | POSIX::O_CREAT(), 0666);
 
     # $from_caller is a sub which converts a function name into one
     # seen from the caller's namespace.  Namespaces do not nest, so if
@@ -94,7 +96,19 @@ sub check_argv( @ ) {
     foreach (@_) {
 	my $r = Memoize::memoize($from_caller->($_),
 				 SCALAR_CACHE => [ HASH => \%cache ],
-				 LIST_CACHE => 'MERGE');
+				 #
+				 # Memoize 1.03 broke tied SCALAR_CACHE with
+				 # together with LIST_CACHE => 'MERGE'. See
+				 # bug report on CPAN:
+				 #
+				 #  https://rt.cpan.org/Public/Bug/Display.html?id=91927
+				 #
+				 # As no user of this module calls memoized
+				 # functions in list context, we can simply
+				 # replace it with 'FAULT'.
+				 #
+				 #LIST_CACHE => 'MERGE');
+				 LIST_CACHE => 'FAULT');
 	die "could not memoize $_" if not $r;
 	push @r, $r;
     }
